// Package util ...
//
// Description : go-jwt具体实现
//
// Author : go_developer@163.com<白茶清欢>
//
// Date : 2023-02-11 20:09
package util

import (
	"errors"

	"github.com/dgrijalva/jwt-go"
)

type ownJwt struct {
}

type MyClaims struct {
	jwt.StandardClaims
}

// Generate 生成 token
//
// Author : go_developer@163.com<白茶清欢>
//
// Date : 20:13 2023/2/11
func (j *ownJwt) Generate(signMethod jwt.SigningMethod, secret string, claims jwt.Claims) (string, error) {
	if nil == signMethod {
		signMethod = jwt.SigningMethodHS256
	}
	var (
		token *jwt.Token
	)

	if nil == claims {
		token = jwt.New(signMethod)
	} else {
		token = jwt.NewWithClaims(signMethod, claims)
	}
	return token.SignedString([]byte(secret))
}

// Validate 验证token
//
// Author : go_developer@163.com<白茶清欢>
//
// Date : 20:13 2023/2/11
func (j *ownJwt) Validate(inputToken string, secret string, convert func(c jwt.Claims) error) error {
	if nil == convert {
		return errors.New("convert func is nil")
	}
	var (
		token *jwt.Token
		err   error
	)

	if token, err = jwt.Parse(inputToken, func(tokenInfo *jwt.Token) (interface{}, error) {
		return []byte(secret), nil
	}); nil != err {
		return errors.New("token parse fail : " + err.Error())
	}
	if !token.Valid {
		return errors.New("token is valid : " + err.Error())
	}
	if nil != convert {
		if err = convert(token.Claims); nil != err {
			return err
		}
	}
	return nil
}
